# %%
import argparse
import subprocess
import re

PYTHON_COMMANDS = {
    "1.13.0": {
        "11.6": "torch torchvision torchaudio --extra-index-url https://download.pytorch.org/whl/cu116",
        "11.7": "torch torchvision torchaudio",
        "cpu": "torch torchvision torchaudio --extra-index-url https://download.pytorch.org/whl/cpu",
        "macOS": "torch torchvision torchaudio"
    },
    "1.12.1": {
        "10.2": "torch==1.12.1+cu102 torchvision==0.13.1+cu102 torchaudio==0.12.1 --extra-index-url https://download.pytorch.org/whl/cu102",
        "11.3": "torch==1.12.1+cu113 torchvision==0.13.1+cu113 torchaudio==0.12.1 --extra-index-url https://download.pytorch.org/whl/cu113",
        "11.6": "torch==1.12.1+cu116 torchvision==0.13.1+cu116 torchaudio==0.12.1 --extra-index-url https://download.pytorch.org/whl/cu116",
        "cpu": "torch==1.12.1+cpu torchvision==0.13.1+cpu torchaudio==0.12.1 --extra-index-url https://download.pytorch.org/whl/cpu",
        "macOS": "torch==1.12.1 torchvision==0.13.1 torchaudio==0.12.1"
    },
}

CUDA_LOOKUP = {}
for pv, v in PYTHON_COMMANDS.items():
    for cv, command in v.items():
        if cv not in CUDA_LOOKUP.keys():
            CUDA_LOOKUP[cv] = []
        CUDA_LOOKUP[cv].append((pv, command))
        CUDA_LOOKUP[cv].sort(key=lambda v: v[0])


def latestValidVersion(cudaVersion):
    goodVersions = list(filter(lambda x: x <= cudaVersion, CUDA_LOOKUP.keys()))
    goodVersions.sort()
    return goodVersions[-1]


def hasCommand(name):
    """
    Check whether `name` is in PATH and is executable 
    """
    from shutil import which
    return which(name) is not None


def getCudaVersion():
    if hasCommand('nvidia-smi'):
        result = subprocess.run(['nvidia-smi'], capture_output=True)
        output = str(result.stdout, encoding='utf-8')
        version = re.search(r'CUDA\s+Version:\s+([\d\.]+)\s+', output)
        try:
            return latestValidVersion(version.group(1))
        except:
            pass
    return 'cpu'


def getPlatform():
    from sys import platform
    return platform
    # if platform == "linux" or platform == "linux2":
    #     # linux
    # elif platform == "darwin":
    #     # OS X
    # elif platform == "win32":
    #     # Windows...


# %%


def main():
    parser = argparse.ArgumentParser()
    args = ['pip', 'install']
    cudaVersion = getCudaVersion()
    if getPlatform() == 'darwin':
        cudaVersion == 'macOS'

    if cudaVersion in ['macOS', 'cpu']:
        print('CPU ONLY')
    else:
        print(f'Found CUDA: {cudaVersion}')

    pipArgs = CUDA_LOOKUP[cudaVersion][-1][1]
    args.extend(re.split(r'\s+', pipArgs))

    print(f'Running Command:\n{" ".join(args)}\n')

    subprocess.run(args=args)


if __name__ == "__main__":
    main()
